# reopen module
module RandomGraph
  # module implements graph algorithms
  module GraphAlgorithms
    def bfs_distance(root)
      distance = Array.new(@size).fill(Float::INFINITY)
      queue = [root]
      distance[root] = 0

      until queue.empty?

        current = queue.pop
        adj = adjacent_nodes(current)
        adj.each do |a|
          if distance[a] == Float::INFINITY
            distance[a] = distance[current] + 1
            queue.unshift(a)
          end
        end
      end
      distance
    end

    def connected_component(root)
      distances = bfs_distance(root)
      adjacent = []
      (0...distances.size).each do |i|
        adjacent << i unless distances[i] == Float::INFINITY
      end
      adjacent
    end

    def number_of_components
      visited = Array.new(size)
      visited.fill(false)
      counter = 0
      (0...size).each do |i|
        next if visited[i] == true
        counter += 1
        comp = connected_component(i)
        comp.each do |reachable|
          visited[reachable] = true
        end
      end
      counter
    end

    def worst_case_diameter
      real_bad = number_of_components > 1 ? Float::INFINITY : 0
      worst = 0
      if real_bad == 0
        (0...size).each do |i|
          distances = bfs_distance(i)
          worst = [distances.max, worst].max
        end
      end
      [worst, real_bad].max
    end

    def average_case_diameter
      real_bad = number_of_components > 1 ? Float::INFINITY : 0
      total = 0
      if real_bad == 0
        (0...size).each do |i|
          distances = bfs_distance(i)
          average = distances.inject(:+).to_f / (size - 1)
          total += average
        end
      end
      total / size
    end

    def clustering_coefficient
      total_cluster = 0
      (0...size).each do |i|
        neighbors = adjacent_nodes(i)
        k = neighbors.size
        possible_connections = k.to_f * (k - 1) / 2
        actual_connections = 0
        (0...k).each do |l|
          (i + 1...k).each do |j|
            actual_connections += 1 if edge?(neighbors[l], neighbors[j])
          end
        end
        if possible_connections == 0 then clustering = 0
        else clustering = actual_connections.to_f / possible_connections
        end
        total_cluster += clustering
      end
      total_cluster / size
    end
  end
end
